searchData={"content_type":"text/markdown","items":[{"doc":"Create and manage a LiveView model. A model allows you to define your LiveView assigns declaratively, similar to how\nyou would use a `Phoenix.Component` and its `attr/2,3` macro.\n\nTo create a model, import this module and use the `defmodel` macro, like so:\n\n```elixir\ndefmodule MyAppWeb.MyLive.Model do\n  import LiveModel\n\n  defmodel do\n    field :user, MyApp.User.t(), required: true\n    field :grocery_list, [MyApp.Food.t()], default: []\n    field :sale_text, String.t() # defaults to `nil`\n  end\nend\n```\n\nThen, in your LiveView module:\n\n```elixir\ndefmodule MyAppWeb.MyLive do\n  use MyAppWeb, :live_view\n\n  alias MyAppWeb.MyLive.Model\n\n  # un-import `assign/2,3` and friends that are included in\n  # `use MyAppWeb, :live_view` to avoid accidental use.\n  # (You should use the model helper functions instead, explained\n  # later in this moduledoc)\n  import Phoenix.Component, except: [assign: 2, assign: 3, assign_new: 3, update: 3]\n  # alternatively, if you don't use other functions in\n  # `Phoenix.Component` and you get an \"unused import\" warning\n  # from the above, you can do something like this instead:\n  # import Phoenix.Component, only: []\n\n  @impl true\n  def mount(_params, %{\"user_id\" => user_id}, socket) do\n    user = MyApp.get_user!(user_id)\n\n    {:ok, Model.assign_new(socket, user)} # assigns a new model struct under the `@model` assign\n  end\n\n  @impl true\n  def handle_event(\"new_sale\", _params, sockt) do\n    # update assigns with `Model.put/2,3` and `Model.update/3`\n    {:noreply, Model.put(socket, :sale_text, \"Apples are now 10% off!\")}\n  end\n\n  @impl true\n  def handle_event(\"some_event\", _params, sockt) do\n    # Dialyzer will warn you when trying to put/update invalid keys\n    {:noreply, Model.put(socket, :bad_key, :uhoh)}\n  end\n```\n\nYou would then access assigns in your render function/template via `@model.assign`, instead of `@assign`.\n\nThe `defmodel` macro will create a struct and `t()` type for you. It will also create the following helper functions:\n- `new/x`: creates a struct, where arity `x` is the number of required fields plus `1`. Required fields are passed\n   as individual arguments to `new/x`, and optional fields are passed in a Keyword list (or another Enumerable like\n   a map)\n- `assign_new/x`: similar to `new/x`, but takes the socket as the first argument and assigns the new struct under\n  `@model`.\n- `put/2-3`: given a LiveView socket, updates the `:model` assign with the given field(s) and value(s). This function\n   is meant to replace use of `Phoenix.Component.assign` in your LiveView\n- `update/2-3`: given a LiveView socket, updates the `:model` assign by passing the current value under `field` to the\n  given `updater` function. The result then replaces the original value. This function\n   is meant to replace use of `Phoenix.Component.update` in your LiveView\n\nPlease read each function's documentation for more information.\n\nMuch of the implementation of `defmodel` is heavily inspired by Lucas San Román's `typedstruct` macro. You can read\nmore about it (and Elixir's AST/macros in general) in their\n[blogpost](https://dorgan.netlify.app/posts/2021/04/the_elixir_ast_typedstruct/).\n\nThe \"model\" naming scheme is inspired by the [Elm architecture/programming language](https://elm-lang.org/).","ref":"LiveModel.html","title":"LiveModel","type":"module"},{"doc":"Define a LiveView model.\n\nThis macro should be given a do block, whose contents are `field`s:\n\n```elixir\ndefmodel do\n  field :my_string_assign, String.t(), default: \"\"\n  field :my_number_assign, integer(), required: true\nend\n```\n\nSee the `LiveModel` documentation for more info and examples.","ref":"LiveModel.html#defmodel/1","title":"LiveModel.defmodel/1","type":"macro"},{"doc":"# LiveModel\n\n**Declarative assigns via structs**","ref":"readme.html","title":"LiveModel","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:live_model, \"~> 0.1.0\"}\n  ]\nend\n```\n\nSee the `LiveModel` module documentation for usage instructions","ref":"readme.html#installation","title":"Installation - LiveModel","type":"extras"}]}